from urllib.parse import urljoin
from dataclasses import dataclass, field

from .api_client import ApiClient
from .api_client import AGENT_WALLET_BASE_URL


@dataclass
class Agent:
    _api_client: ApiClient = field(repr=False, compare=False)
    endpoint_id: str
    name: str
    description: str
    author: str
    price_usd_cents: int

    @property
    def base_url(self):
        return urljoin(AGENT_WALLET_BASE_URL, f"agents/{self.name}")



from typing import List, Optional
from dataclasses import dataclass, field

from .api_client import ApiClient
from .transaction import Transaction


@dataclass
class Card:
    card_name: str
    card_number: str
    expiry: str
    cvv: str


@dataclass
class Wallet:
    _api_client: ApiClient = field(repr=False, compare=False)
    wallet_uid: str
    transactions: Optional[List[Transaction]] = None
    balance_usd_cents: Optional[int] = None

    def transfer(self, transfer_to: str, amount_usd_cents: int) -> bool:
        data = {
            "transfer_to_user_email": transfer_to,
            "amount_usd_cents": amount_usd_cents,
        }
        response = self._api_client.post(
            f"wallet/{self.wallet_uid}/transfer", data=data
        )
        return response["response"] == "OK"

    def balance(self) -> int:
        response = self._api_client.get(f"wallets/{self.wallet_uid}")
        self.balance_usd_cents = response["wallet"]["balance_usd_cents"]
        return self.balance_usd_cents

    def get_card(self, agent_name: str) -> Optional[Card]:
        url = f"wallets/{self.wallet_uid}/card?card_name={agent_name}"
        response = self._api_client.get(url)
        if response:
            card_dict = response["card"]
            return Card(
                card_name=card_dict["card_name"],
                card_number=card_dict["card_number"],
                expiry=card_dict["expiry"],
                cvv=card_dict["cvv"],
            )



class InvalidCredentialsException(Exception):
    def __init__(self, message="Invalid credentials"):
        super().__init__(message)


class APIException(Exception):
    def __init__(self, error_info):
        super().__init__(error_info)



import os
import requests
from urllib.parse import urljoin
from .exceptions import APIException, InvalidCredentialsException

AGENT_WALLET_BASE_URL = os.getenv(
    "AGENT_WALLET_BASE_URL", "https://testwallet.sidekik.ai"
)
REQUEST_TIMEOUT = 10  # seconds


class ApiClient:
    def __init__(self, api_key: str) -> None:
        self.api_key = api_key
        self.headers = {"Authorization": f"Bearer {api_key}"}

    def get(self, url: str) -> dict:
        response = requests.get(
            urljoin(AGENT_WALLET_BASE_URL, url),
            headers=self.headers,
            timeout=REQUEST_TIMEOUT,
        )
        return self._handle_response(response)

    def post(self, url: str, data: dict) -> dict:
        response = requests.post(
            urljoin(AGENT_WALLET_BASE_URL, url),
            headers=self.headers,
            json=data,
            timeout=REQUEST_TIMEOUT,
        )
        return self._handle_response(response)

    def _handle_response(self, response: requests.Response) -> dict:
        if response.ok:
            return response.json()
        else:
            self._handle_error(response)

    def _handle_error(self, response: requests.Response):
        if response.status_code == 401:
            raise InvalidCredentialsException()
        elif response.status_code == 400:
            data = response.json()
            if data.get("response") == "NOK" and "error" in data:
                raise APIException(error_info=data["error"])
        response.raise_for_status()  # Reraises the original HTTPError if not handled above



from typing import List
from typing import Optional

from .agent import Agent
from .wallet import Wallet
from .transaction import Transaction
from .api_client import ApiClient


class Account:
    def __init__(self, api_key: str) -> None:
        self.api_client = ApiClient(api_key)

    @classmethod
    def from_key(cls, api_key: str) -> "Account":
        return cls(api_key)

    def get_wallet(self, wallet_uid: str) -> Optional[Wallet]:
        response = self.api_client.get(f"wallets/{wallet_uid}")
        if response.get("wallet"):
            transactions = []
            raw_transactions = response["wallet"].pop("transactions")
            for rt in raw_transactions:
                transactions.append(Transaction(**rt))
            return Wallet(
                _api_client=self.api_client,
                transactions=transactions,
                **response["wallet"],
            )

    def get_wallets(self) -> List[Wallet]:
        response = self.api_client.get("wallets")
        wallets = []
        for w in response["wallets"]:
            wallets.append(Wallet(_api_client=self.api_client, **w))
        return wallets

    def get_agents(self) -> List[Agent]:
        response = self.api_client.get("agents")
        agents = []
        for w in response["agents"]:
            agents.append(Agent(_api_client=self.api_client, **w))
        return agents
